/*
 * Created on 2004/09/02
 *
 */
package jnet.hirommy.core;

import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.info.HirommyPacketFactory;
import jnet.hirommy.util.InfoLogger;
import jnet.hirommy.util.ViewerHandler;
import jnet.hirommy.util.ViewerHandlerImpl;

/**
 * This class processes sending process.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class SendingProcess {
	
	private static HirommyPacketFactory hirommyPacketFactory = new HirommyPacketFactory();
	
	/**
	 * This method provide function of sending process.
	 * First, HirommyPacket for sending other terminal is created from GGA data.
	 * Second, it is logged to InfoLogger.
	 * Therd, it is sended by UDP. 
	 * But, if this system use Reflect, this process don't send HirommyPacket.
	 * Finally, update Viewer.
	 * 
	 * @param ggaData GGA data from satelaite.
	 */
	public static void processSending(String ggaData) {
		
		// create hirommy packet form GGA data.
		HirommyPacket hirommyPacket = hirommyPacketFactory.createHirommyPacket(ggaData);
		
		// hirommyPacket is saved at InfoLogger.
		InfoLogger.getInstance().logMyHirommyPacket(hirommyPacket);
		
		// The Hirommy Packet will be transmitted if Refrect mode is ON.
		// Otherwise, The Hirommy Packet does not transmit.
		if (!SystemProperty.getInstance().isReflect()) {
			InfoSender.sendPacket(hirommyPacket);
		}
		
		// update Viewer.
		ViewerHandler viewerHandler = new ViewerHandlerImpl();
		viewerHandler.repaintGpsViewer();							// update GpsViewer.
		viewerHandler.repaintMyPropertyViewer();					// update MyPropetyViewer.
		
		System.out.println("Sending : " + hirommyPacket.toString());	// show created infomation.
	}
}
