/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.info;

import java.util.LinkedList;
import java.util.StringTokenizer;

import jnet.hirommy.core.SystemProperty;
import jnet.hirommy.util.InfoLogger;
import jnet.its.gps.AccelCalculator;
import jnet.its.gps.HeaddingCalculator;
import jnet.its.gps.SimpleAccelCalculationMethod;
import jnet.its.gps.SimpleHeaddingCalculationMethod;
import jnet.its.gps.SimpleSpeedCalculationMethod;
import jnet.its.gps.SpeedCalculator;

/**
 * This class for creating GPS infomation.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class HirommyPacketFactory {
	
	// TODO set up algorithm for option infomation.
	/**
	 * algorithom for calculating speed.
	 */
	private static SpeedCalculator speedCalculator =
			new SimpleSpeedCalculationMethod();
	
	/**
	 * algorithom for calculating accel.
	 */
	private static AccelCalculator accelCalculator =
			new SimpleAccelCalculationMethod();
	
	/**
	 * algorithom for calculating headding.
	 */
	private static HeaddingCalculator headdingCalculator =
			new SimpleHeaddingCalculationMethod();
	
	
	/**
	 * This method for creating HirommyPcket.
	 * 
	 * @param ggaData GPS infomation.
	 * @return hirommy packet.
	 */
	public HirommyPacket createHirommyPacket(String ggaData) {
		
		SystemProperty myProperty = SystemProperty.getInstance();	// get MyProperty instance.
		
		String addr = myProperty.getAddr();							// this terminal IP Address.
		MyProfileImpl myProfile = MyProfileImpl.getInstance();		// get MyProfile.
		GpsInfo gpsInfo = createGpsInfo(ggaData);					// GPS infomation.
		OptionInfo optionData = createOptionInfo(gpsInfo);			// option data.
		EventInfo eventInfo = createEventInfo();					// Event infomation.
		
		// create Hirommy Packet to send.
		HirommyPacket hirommyPacket = new HirommyPacket(addr,
														myProfile,
														gpsInfo,
														optionData,
														eventInfo);
		
		return hirommyPacket;
	}
	
	/**
	 * This method generates HirommyPacket from GPS information.
	 * This method is used when inputting data from an XML file.
	 * 
	 * @param ipAddr IPaddress.
	 * @param profile target profile.
	 * @param gpsInfo GPS infomation.
	 * @return hirommy packet.
	 */
	public HirommyPacket createHirommyPacket(String ipAddr,
											  MyProfile profile,
											  GpsInfo gpsInfo,
											  OptionInfo optionInfo,
											  EventInfo eventInfo) {
		SystemProperty myProperty = SystemProperty.getInstance();	// get MyProperty instance.
		
		// create Hirommy Packet to send.
		HirommyPacket hirommyPacket = new HirommyPacket(ipAddr,
														profile,
														gpsInfo,
														optionInfo,
														eventInfo);
		
		return hirommyPacket;
	}
	
	/**
	 * This method is changed into the format used by Hirommy network from a GGA format.
	 * transform from String of GGA format to Hirommy format.<br>
	 * 
	 * String of GGA format<br>
	 * GGA - Global Positioning System Fix Data<br>
	 * GGA,123519.00,4807.038247,N,01131.324523,E,1,08,0.9,545.42,M,46.93,M,5.0,1012*42<br>
	 * 
	 * 1 : time<br>
	 * 2 : latitude<br>
	 * 4 : longitude<br>
	 * 5 : position state<br>
	 * 6 : satellite<br>
	 * 11: altitude<br>
	 * 
	 * @param ggaData GGA date.
	 * @return GpsInfo GPS infomation.
	 */
	private GpsInfo createGpsInfo(String ggaData) {
		
		String time = null;			// time
		String latitude = null;		// latitude
		String longitude = null;	// longitude
		String state = null;		// position state
		String satellite = null;	// the number of satellite
		String altitude = null;		// altitude
		
		StringTokenizer st = new StringTokenizer(ggaData, ",");
		
		// sampling time, latitude and longitude from GGA format.
		for(int i = 0; st.hasMoreTokens() == true; i++) {
			if(i == 1) {
				time = st.nextToken(); 		// time
			} else if (i == 2) {
				latitude = st.nextToken();	// latitude
			} else if (i == 4) {
				longitude = st.nextToken();	// longitude
			} else if (i == 6) {
				state = st.nextToken();		// position stete
			} else if (i == 7) {
				satellite = st.nextToken();	// the number of satellite
			} else if (i == 11) {
				altitude = st.nextToken();	// altitude
			} else {
				st.nextToken();
			}
		}
		
		return new GpsInfo(time,
							latitude,
							longitude,
							state,
							satellite,
							altitude);
	}
	/**
	 * This method for creating Option infomation.
	 * 
	 * @gpsInfo current GPS infomation.
	 * @return OptionInfo.
	 */
	private OptionInfo createOptionInfo(GpsInfo gpsInfo) {
		LinkedList lists = InfoLogger.getInstance().getMyInfo();
		
		// calculate speed.
		String speed = speedCalculator.calculateSpeed(lists, gpsInfo);
		
		// calculate headding.
		String headding = headdingCalculator.calculateHeadding(lists, gpsInfo);
		
		// calculate accel.
		String accel = accelCalculator.calculateAccel(lists, gpsInfo, speed);
		
		return new OptionInfo(accel,
							   speed,
							   headding);
	}
	/**
	 * This method for creating Event infomation.
	 * 
	 * @return EventInfo.
	 */
	private EventInfo createEventInfo() {
		// TODO code a method for creating Option infomation.
		return new EventInfo();
	}
}
