/*
 * Created on 2004/09/01
 *
 */
package jnet.hirommy.util.file;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import jnet.hirommy.info.EventInfo;
import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.info.HirommyPacketFactory;
import jnet.hirommy.info.OptionInfo;
import jnet.hirommy.info.TargetProfileImpl;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

/**
 * This class implements XMLDecoder interface.
 * This class consider syncronization.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class SimpleXMLDecoder implements XMLDecoder {

	/**
	 * This method decode XML fail.
	 * This method consider syncronization.
	 * 
	 * @param fileName file name.
	 * @see jnet.hirommy.util.file.XMLDecoder#decode(java.lang.String)
	 */
	public LinkedList decode(String fileName) {
		
		LinkedList lists = new LinkedList();		// lists of Hirommy packet.
		
		String ipAddr = null;						// IP Address
		String time = null;							// time
		String latitude = null;						// latitude
		String longitude = null;					// longitude
		String state = null;						// state
		String satellite = null;					// number of satellite
		String altitude = null;						// altitude
		
		String userName = null;						// user name
		int authorizeType = 0;						// authorize type
		int terminalType = 0;						// terminal type
		
		String accel = "0";							// accel
		String speed = "0";							// speed
		String headding = "0";						// headding
		
		// factory of Hirommy Packet.
		HirommyPacketFactory hirommyPacketFactory = new HirommyPacketFactory();
		
		try {
			// get reader object.
			BufferedReader reader = 
					new BufferedReader(new FileReader(fileName));
			
			String buff = null;
			while ((buff = reader.readLine()) != null) {
			
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = factory.newDocumentBuilder();
				
				ByteArrayInputStream byteArrayStream = new ByteArrayInputStream(buff.getBytes());
				
				Document doc = builder.parse(byteArrayStream);
				
				Element root = doc.getDocumentElement();
				
				NodeList infoRoot = root.getChildNodes();
				
				// get IPAddress.
				Node ipAddressChild = infoRoot.item(0);
				if (ipAddressChild instanceof Element) {
					Element childElement = (Element)ipAddressChild;
					Text textNode = (Text)childElement.getFirstChild();
					String value = textNode.getData().trim();
					if (childElement.getTagName().equals("IPAddress")) {
						// get IPAddress
						ipAddr = value;
					}
				}
				
				// get other infomation.
				for (int i = 1; i < infoRoot.getLength(); i++) {
					// infoChild is <MyProfile>, <GPSInfomation>, <OptionInfo> and <EventInfo> 
					Node infoChild = infoRoot.item(i);
					NodeList infoChildLists = infoChild.getChildNodes();
					for (int j = 0; j < infoChildLists.getLength(); j++) {
						Node infoElement = infoChildLists.item(j);
						if (infoElement instanceof Element) {
							Element childElement = (Element)infoElement;
							Text textNode = (Text)childElement.getFirstChild();
							String value = textNode.getData().trim();
							if (childElement.getTagName().equals("time")) {
								// get time
								time = value;
							} else if (childElement.getTagName().equals("latitude")) {
								// get latitude
								latitude = value;
							} else if (childElement.getTagName().equals("longitude")) {
								// get longitude
								longitude = value;
							} else if (childElement.getTagName().equals("state")) {
								// get state
								state = value;
							} else if (childElement.getTagName().equals("satellite")) {
								// get number of sttellite
								satellite = value;
							} else if (childElement.getTagName().equals("altitude")) {
								// get altitude
								altitude = value;
							} else if (childElement.getTagName().equals("UserName")) {
								// get user name
								userName = value;
							} else if (childElement.getTagName().equals("TerminalType")) {
								// get terminal type
								terminalType = Integer.parseInt(value);
							} else if (childElement.getTagName().equals("AuthorizeType")) {
								// get authorize type
								authorizeType = Integer.parseInt(value);
							} else if (childElement.getTagName().equals("Accelerator")) {
								// get accel
								accel = value;
							} else if (childElement.getTagName().equals("Speed")) {
								// get speed
								speed = value;
							} else if (childElement.getTagName().equals("Headding")) {
								// get headding
								headding = value;
							}
						}
					}
				}
				
				byteArrayStream.close();		// close byteArrayInputStream.
				
				// create HirommyPacket.
				HirommyPacket hirommyPacket = hirommyPacketFactory.createHirommyPacket(
																ipAddr,
																new TargetProfileImpl(userName,
																					  authorizeType,
																					  terminalType),
																new GpsInfo(time,
																			latitude,
																			longitude,
																			state,
																			satellite,
																			altitude),
																new OptionInfo(accel,
																				speed,
																				headding),
																new EventInfo());
				
				lists.add(hirommyPacket);
			}
			
			reader.close();						// close buffered stream.
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		}
		
		return lists;
	}

}
