/*
 * Created on 2004/08/29
 *
 */
package jnet.its.gps;

import java.util.LinkedList;

import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;

/**
 * This is a class which realizes a simple headding calculation method.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class SimpleHeaddingCalculationMethod extends HeaddingCalculatorImpl {

	/**
	 * This is a simple headding calculation method
	 * 
	 * 
	 * @param lists Sauce information for calculation headding
	 * @param currentGpsInfo currentGpsInfo lately GpsInfomation.
	 * @return headding direction.
	 * @see jnet.its.gps.HeaddingCalculator#calculateHeadding(java.util.LinkedList, jnet.hiromy.info.GpsInfo)
	 */
	public String calculateHeadding(LinkedList lists, GpsInfo currentGpsInfo) {
		if (lists.size() >= 2) {
			// The last element is taken out.
			// / The GPS information of 1 second ago is taken out.
			GpsInfo exGpsInfo = ((HirommyPacket)lists.getLast()).getGpsInfo();
			
			// calculate the direction.
			String direction = getDirction(currentGpsInfo.getLatitude(),
													   currentGpsInfo.getLongitude(),
													   exGpsInfo.getLatitude(),
													   exGpsInfo.getLongitude());
			
			return direction;
			
		} else {	// When there is no GPS information for calculating speed.
			return null;
		}
	}
	
	/**
	 * This method return direction of terminal.
	 * 
	 * @param currentLatitude current Latitude
	 * @param currentLongitude current Longitude
	 * @param exLatitude Latitude of 1 second befor
	 * @param exLongitude Longitude of 1 second befor.
	 * 
	 * @return direction.
	 */
	private String getDirction(String currentLatitude,
								String currentLongitude,
								String exLatitude,
								String exLongitude) {
		
		// This is terminal current position.
		double ty = Double.parseDouble(currentLatitude);
		double tx = Double.parseDouble(currentLongitude);
		
		// This is terminal current position of 1 second befor
		double ey = Double.parseDouble(exLatitude);
		double ex = Double.parseDouble(exLongitude);
		
		double dx = tx - ex;	// distance between terminal position and event position.
		double dy = ty - ey;
		
		double radian = Math.atan2(dy, dx);
		double degree = Math.toDegrees(radian);
		
		double compassDegree = 90.0 -degree;
		if (compassDegree < 0.0){
			compassDegree +=360;
		}
		
		return String.valueOf(compassDegree);
	}
}
