/*
 * Created on 2004/09/03
 *
 */
package jnet.its.ivc.predict;

import java.util.LinkedList;

import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;

/**
 * This class mounts concrete processing which calculates a prediction value. 
 * The simple prediction method is offered in this class.
 * The simple prediction method is the method 
 * that calcule future prediction values from for the past two points.
 * A prediction value is calculated until after 5 seconds.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class SimplePredictionMethod implements PredictionAlgorithm {

	/**
	 * predict after 5 second.
	 */
	private static final int PREDICTED_TIME = 5;
	
	/**
	 * This method calculates the prediction value of a terminal. 
	 * As for the link list of return values, 
	 * the prediction value is contained in order of A
	 * after [ of a head ] 1 second, 2 seconds....
	 * 
	 * @param lists The past position information list.
	 * @return predict position of terminal.
	 * @see jnet.its.ivc.predict.PredictionAlgorithm#predict()
	 */
	public LinkedList predict(LinkedList lists) {
		if (lists.size() >= 2) {
			
			// get current GPS infomation.
			GpsInfo currentGpsInfo = ((HirommyPacket)lists.getLast()).getGpsInfo();
			// get ex GPS infomation.
			GpsInfo exGpsInfo = ((HirommyPacket)lists.get(lists.size() - 2)).getGpsInfo();
			
			// get distance of latitude between current latitude and ex latitude.
			double dy = Double.parseDouble(currentGpsInfo.getLatitude())
						 - Double.parseDouble(exGpsInfo.getLatitude());
			
			// get distance of latitude between current longitude and ex longitude.
			double dx = Double.parseDouble(currentGpsInfo.getLongitude())
						 - Double.parseDouble(exGpsInfo.getLongitude());
			
			// if terminal don't move, don't prediction.
			if (dy == 0 && dx ==0) {
				return null;
			}
			
			// create prediction position.
			LinkedList predictedPositionLists = new LinkedList();
			for (int i = 0; i < 5; i++) {
				predictedPositionLists.addLast(
						new PredictPosition(Double.parseDouble(currentGpsInfo.getLatitude()) + dy * (i + 1),
									 		 Double.parseDouble(currentGpsInfo.getLongitude()) + dx * (i + 1)));
			}
			
			return predictedPositionLists;
		} else {
			// It will not predict, 
			// if the information for calculating a prediction value does not have two or more.
			return null;
		}
	}

}
